import numpy as np
import matplotlib.pyplot as plt
from srenet.data import sample_data
from srenet.projgrad import projgrad
from srenet.utils import ProjgradParams

m, n = 100, 300                 # Size of A
gen_dic = 'gaussian'            # Sampling method for A
gen_obs = 'gaussian'            # Sampling method for y
l1ratio = 0.1                   # l1-regularizer / max(A.T @ y)
l2ratio = 0.1                   # l2-regularizer / max(A.T @ y)

x0 = np.zeros(n)
A, y = sample_data(m, n, gen_dic, gen_obs)
lmax = np.max(A.T @ y)
l1 = l1ratio * lmax
l2 = l2ratio * lmax

params = ProjgradParams(verbosity=False, screen=True, relax=True)
x, mnt = projgrad(x0, A, y, l1, l2, params)
I_pos = np.sum(np.abs(x) > 1e-8)
I_null = np.sum(np.abs(x) <= 1e-8)

print("-------------------")
print("Identification test\n")
print("aPGsr")
print("  Stop crit       : {}".format(mnt.stopcrit))
print("  Last Gap        : {:.2e}".format(mnt.gap[-1]))
print("  Last Obj        : {:.2e}".format(mnt.obj[-1]))
print("  Iters           : {}".format(len(mnt.gap)))
print("  FLOPS           : {:.2e}".format(mnt.flops[-1]))
print("Sparisty of the solution : {}/{} ({:.2%})".format(I_pos, n, I_pos/n))

fig, ax = plt.subplots(1,1,figsize=(4,3))

ax.plot(mnt.gap, mnt.sc_size/I_null, label="% Screened", marker='.')
ax.plot(mnt.gap, mnt.rc_size/I_pos, label="% Relaxed", marker='.')
ax.yaxis.set_major_formatter(lambda x,pos: r"{0:.0f}%".format(x * 100))
ax.invert_xaxis()
ax.set_xlabel("GAP")
ax.set_ylabel("Indices identified")
ax.set_xscale('log')
ax.grid(b=True, which='major', axis='y')
ax.legend()
plt.tight_layout()
plt.show()

print("-------------------")